! ****************************************************************************
!                Time-stamp: <2014-02-13 12:05:31 sander>
!     Author: Rolf Sander, Max-Planck Institute, Mainz, Germany, 2009-2014
!*****************************************************************************

! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License
! as published by the Free Software Foundation; either version 2
! of the License, or (at your option) any later version.
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! You should have received a copy of the GNU General Public License
! along with this program; if not, get it from:
! http://www.gnu.org/copyleft/gpl.html

! ****************************************************************************

MODULE jvpp_mem

  USE messy_cmn_photol_mem, ONLY: IP_MAX
  IMPLICIT NONE

  CHARACTER(LEN=*), PARAMETER :: HLINE1 = &
    '************************************************************************'
  CHARACTER(LEN=*), PARAMETER :: HLINE2 = &
    '------------------------------------------------------------------------'
  CHARACTER(LEN=*), PARAMETER :: HLINE2b = &
    '------------------------------------------------------------------'

  ! temporary (workdir) directories:
  CHARACTER(LEN=*), PARAMETER :: DIR_176 = 'workdir_176'
  CHARACTER(LEN=*), PARAMETER :: DIR_EFF = 'workdir_eff'
  CHARACTER(LEN=*), PARAMETER :: DIR_F90 = 'workdir_f90'
  CHARACTER(LEN=*), PARAMETER :: DIR_JNL = 'workdir_jnl'

  ! input directory ("dat_m17" or "dat_lit"):
  CHARACTER(LEN=7) :: inputdir

  ! l_* logicals showing which input files are available:
  LOGICAL, DIMENSION(IP_MAX) :: l_sig     = .FALSE.
  LOGICAL, DIMENSION(IP_MAX) :: l_s2t     = .FALSE.
  LOGICAL, DIMENSION(IP_MAX) :: l_s3t     = .FALSE.
  LOGICAL, DIMENSION(IP_MAX) :: l_phi     = .FALSE.
  LOGICAL, DIMENSION(IP_MAX) :: l_tc1     = .FALSE.
  LOGICAL, DIMENSION(IP_MAX) :: l_tc2     = .FALSE.
  LOGICAL, DIMENSION(IP_MAX) :: l_tc3     = .FALSE.
  LOGICAL, DIMENSION(IP_MAX) :: l_species = .FALSE.
  LOGICAL, DIMENSION(IP_MAX) :: l_tdep    = .FALSE.

  ! IO-units:
  INTEGER, PARAMETER :: IO_LOG = 99 ! logfile
  INTEGER, PARAMETER :: IO_TMP = 98 ! temporary files

  ! IO-units for step 1:
  INTEGER, PARAMETER :: IO_JNL = 11 ! jnl
  INTEGER, PARAMETER :: IO_SIG = 12 ! *.sig
  INTEGER, PARAMETER :: IO_176 = 13 ! *.176

  ! IO-units for step 2:
  INTEGER, PARAMETER :: IO_IN  = 21 ! input files
  INTEGER, PARAMETER :: IO_OUT = 22 ! output files
  INTEGER, PARAMETER :: IO_TDP = 23 ! temperature-dependent spectra

  ! IO-units for step 3:
  INTEGER, PARAMETER :: IO_JTC = 32 ! jnl, T-const
  INTEGER, PARAMETER :: IO_JTD = 33 ! jnl, T-dep
  INTEGER, PARAMETER :: IO_ERR = 34 ! error in %
  INTEGER, PARAMETER :: IO_REC = 35 ! recalc
  INTEGER, PARAMETER :: IO_F90 = 36 ! jval_*.f90
  INTEGER, PARAMETER :: IO_NML = 37 ! nml file
  INTEGER, PARAMETER :: IO_CAT = 38 ! cat script
  INTEGER, PARAMETER :: IO_CAL = 39 ! jval_cal subroutine
  INTEGER, PARAMETER :: IO_TEX = 40 ! LaTeX file

  INTEGER, PARAMETER :: NWAV = 176
  INTEGER, PARAMETER :: NBIN = 8
  ! wavelengths [nm] in the middle of the intervals (partially defined
  ! by Bruehl & Crutzen, Clim. Dyn. 2, 173-203, 1988, ref2635)
  ! DO jw=1,13
  !   wave_nm(jw) = 1E7/(56250.-500.*jw)
  ! ENDDO
  ! DO jw=14,45
  !   wave_nm(jw) = 1E7/(49750.-(jw-13)*500.)
  ! ENDDO
  ! DO jw=46,68
  !   wave_nm(jw) = (266.+(jw-13))
  ! ENDDO
  ! DO jw=69,71
  !   wave_nm(jw) = (320.5+2.*(jw-68))
  ! ENDDO
  ! DO jw=72,NWAV
  !   wave_nm(jw) = (325.+5.*(jw-71))
  ! ENDDO
  REAL, PARAMETER :: wave_nm(NWAV) = (/ &
    179.37, 181.00, 182.65, 184.33, 186.05, 187.79, 189.57, 191.39, 193.24, &
    195.12, 197.04, 199.00, 201.01, 203.05, 205.13, 207.25, 209.42, 211.64, &
    213.90, 216.22, 218.58, 220.99, 223.46, 225.99, 228.57, 231.21, 233.92, &
    236.69, 239.52, 242.42, 245.40, 248.45, 251.57, 254.78, 258.06, 261.44, &
    264.90, 268.46, 272.11, 275.86, 279.72, 283.69, 287.77, 291.97, 296.30, &
    299.00, 300.00, 301.00, 302.00, 303.00, 304.00, 305.00, 306.00, 307.00, &
    308.00, 309.00, 310.00, 311.00, 312.00, 313.00, 314.00, 315.00, 316.00, &
    317.00, 318.00, 319.00, 320.00, 321.00, 322.50, 324.50, 326.50, 330.00, &
    335.00, 340.00, 345.00, 350.00, 355.00, 360.00, 365.00, 370.00, 375.00, &
    380.00, 385.00, 390.00, 395.00, 400.00, 405.00, 410.00, 415.00, 420.00, &
    425.00, 430.00, 435.00, 440.00, 445.00, 450.00, 455.00, 460.00, 465.00, &
    470.00, 475.00, 480.00, 485.00, 490.00, 495.00, 500.00, 505.00, 510.00, &
    515.00, 520.00, 525.00, 530.00, 535.00, 540.00, 545.00, 550.00, 555.00, &
    560.00, 565.00, 570.00, 575.00, 580.00, 585.00, 590.00, 595.00, 600.00, &
    605.00, 610.00, 615.00, 620.00, 625.00, 630.00, 635.00, 640.00, 645.00, &
    650.00, 655.00, 660.00, 665.00, 670.00, 675.00, 680.00, 685.00, 690.00, &
    695.00, 700.00, 705.00, 710.00, 715.00, 720.00, 725.00, 730.00, 735.00, &
    740.00, 745.00, 750.00, 755.00, 760.00, 765.00, 770.00, 775.00, 780.00, &
    785.00, 790.00, 795.00, 800.00, 805.00, 810.00, 815.00, 820.00, 825.00, &
    830.00, 835.00, 840.00, 845.00, 850.00 /)

END MODULE jvpp_mem

! ****************************************************************************
